/////////////////////////////////////////////////////////////////////////////
//
//         DLL API for RDS signal generator "HG 81300-B/PC" v1.0
//        ------------------------------------------------------
//
//   application programing interface (API) for dynamic link library (DLL):
//
//   remote control of RDS-coder "HG 81300-B/PC" via serial COM port
//   for Microsoft Windows 32 bit, developed with Microsoft Visual C++ 6.0
//
//   (C) 2001 by Gtting KG, radio sets   Author: O. Herbst, Feb. 2001
//
//   address: Gtting KG, Celler Strae 5, 31275 Lehrte/Rddensen, Germany
//   http:    www.goetting.de
//   email:   RDS-Support@Goetting.de
//
/////////////////////////////////////////////////////////////////////////////


#if !defined(RDSHG81300B_H_INCLUDED)
#define      RDSHG81300B_H_INCLUDED

#ifdef  RDSHG81300B_EXPORTS
#define RDSHG81300B_API __declspec(dllexport)
#else
#define RDSHG81300B_API __declspec(dllimport)
#endif


//---> Note:  All identifiers refering to this DLL begin with "HG_".


//---------- Constants for values -------------------------------------------

#define HG_SLOT_NUMBER_MIN         1  // lowest 'program slot' number

#define HG_SLOT_NUMBER_MAX        63  // highest 'program slot' number

#define HG_NO_SLOT                 0  // no 'program slot' activated

#define HG_SLOT_TIMES_MAX         99  // maximum number of times a 'program
                                      // slot' can be sent
#define HG_SLOT_TIMES_INFINITE     0  // sending times infinite

#define HG_FREQ_MIN             87.5  // minimum FM frequency in MHz
#define HG_FREQ_MAX            108.0  // maximum FM frequency in MHz

#define HG_MEMORY_MAX_BYTES    30238  // maximum memory in the RDS-coder free
#define HG_MEMORY_MAX_GROUPS    2326  //  (number of bytes resp. RDS groups)

#define HG_NAME_LEN_MAX            8  // max. number of characters for a
                                      //  program slot name

//---------- constants for error codes returned by the functions ------------

#define HG_ERR_NO           0  // no error
#define HG_ERR_INVALID_ARG -1  // a function argument has an invalid value
#define HG_ERR_NOT_INIT    -2  // COM port not initialized
#define HG_ERR_WRITING     -3  // error occurred while writing via com port
#define HG_ERR_READING     -4  // error occurred while reading via com port
                               // (time out, wrong character)
#define HG_ERR_MEMORY      -5  // RDS-coder is out of memory space.
#define HG_ERR_CHECK_DEL   -6  // 'program slot' is still occupied.
#define HG_ERR_CHECK_STORE -7  // incorrect data programmed in 'program slot'
#define HG_ERR_CHECK_SEL   -8  // 'program slot' is empty, nothing to send or
                               //  delete


//---------- data structures ------------------------------------------------

typedef struct       //---- data set of a 'program slot' in the RDS-coder
{
    char strname [HG_NAME_LEN_MAX+1];  
                     // name of the RDS program (byte sequence)
                     //  up to 8 characters, no space character
    int       slot;  // number of the 'program slot', value range:
                     //  HG_SLOT_NUMBER_MIN...HG_SLOT_NUMBER_MAX
    int      bytes;  // program length in bytes, value range:
                     //  13...HG_MEMORY_MAX_BYTES
    int     groups;  // program length as number of RDS groups, value range:
                     //   1...HG_MEMORY_MAX_GROUPS
    int      times;  // how often the program is transmitted, value range:
                     //   1...HG_SLOT_TIMES_MAX or HG_SLOT_TIMES_INFINITE
    int  next_slot;  // number of the following 'program slot', jumped to
                     //  after having sent this one, value range:
                     //  HG_NO_SLOT, HG_SLOT_NUMBER_MIN...HG_SLOT_NUMBER_MAX
} sHG_ProgSlot;

enum enmHG_switches  //---- all switches in the RDS-coder
{
    HG_FREQ_A,       // transmit on frequency A
    HG_FREQ_B,       // transmit on frequency B ("not HG_FREQ_A")
    HG_SK,           // ARI: information carrier
    HG_BK,           // ARI: traffic area identification
    HG_DK,           // ARI: traffic announcement identification
    HG_PILOT,        // pilot tone (19 kHz)
    HG_STEREO,       // stereo multiplexer
    HG_ST1L,         // left  stereo tone (1900 Hz) and
    HG_ST1R,         // right stereo tone (4750 Hz) for
                     //  the internal modulator
    HG_ST2L,         // left  stereo tone ( 678 Hz) and
    HG_ST2R,         // right stereo tone (3160 Hz) for
                     //  the external modulator
};

typedef struct       //---- state of the RDS-coder
{                    //                         value -1 = "still unknown"
    int    Slot;     // number of the 'program slot' currently being sent
                     //  (HG_SLOT_NUMBER_MIN...HG_SLOT_NUMBER_MAX,
                     //   HG_NO_SLOT: no slot active)

    double fFreqA;   // value of frequency A, MHz (HG_FREQ_MIN...HG_FREQ_MAX)
    double fFreqB;   // value of frequency B, MHz (HG_FREQ_MIN...HG_FREQ_MAX)

                     // state of all switches in the RDS-coder
                     //  (see enmHG_Switches)
    int    FreqA;    // 1 = "frequency A active", 0 = "frequency B active"

                     // for the following variables see enmHG_Switches
    int    SK;       // values:  1 = "on"
    int    BK;       //          0 = "off"
    int    DK;       //         -1 = "still unknown / not to change"
    int    Pilot;
    int    Stereo;
    int    ST1L;
    int    ST1R;
    int    ST2L;
    int    ST2R;
} sHG_state;


//---------- API functions --------------------------------------------------

RDSHG81300B_API int __stdcall  HG_Open  (int number);
//
// opens the serial COM port, defines parameters (9600 bps, 8n1),
// data buffers and time out (2 sec).
//
// Input:  number - com port number (1 for COM1, 2, ... 8)
//
// Output: HG_ERR_NO, HG_ERR_INVALID_ARG, HG_ERR_NOT_INIT
 

RDSHG81300B_API int __stdcall  HG_Close  (void);
//
// closes the serial COM port (automatically executed while detaching the DLL).
//
// Input:  -
//
// Output: HG_ERR_NO, HG_ERR_NOT_INIT, HG_ERR_WRITING


RDSHG81300B_API int __stdcall  HG_Reset  (void);
//
// resets the RDS-coder and restores the default program slot settings
// and frequencies.
//
// [To restore the non-PC-mode hold down both frequency keys
//  while switching on the RDS-coder. ]
//
// Input:  -
//
// Output: HG_ERR_NO, HG_ERR_NOT_INIT, HG_ERR_WRITING, EG_ERR_READING


RDSHG81300B_API int __stdcall  HG_GetState  (sHG_state *state);
//
// gets the states of the RDS-coder's switches and
// sets them in the given structure.
//
// Input:  state - pointer to state structure
//
// Output: HG_ERR_NO, HG_ERR_INVALID_ARG, HG_ERR_NOT_INIT,
//         HG_ERR_WRITING, EG_ERR_READING


RDSHG81300B_API int __stdcall  HG_SetState  (sHG_state state);
//
// sets the states of the RDS-coder's switches given in the structure.
// If an element in the structure is set to a negative value,
// the corresponding switch will not be changed.
//
// Input:  state - state structure
//
// Output: HG_ERR_NO, HG_ERR_INVALID_ARG, HG_ERR_NOT_INIT, 
//         HG_ERR_WRITING, EG_ERR_READING


RDSHG81300B_API int __stdcall  HG_GetProgList  (sHG_ProgSlot** list);
//
// The DLL contains an array of sHG_ProgSlot with all occupied 'program slots'.
// The array has got up to HG_SLOT_NUMBER_MAX entries.
// The entries are sorted by their slot number in ascending order.
// This function copies the pointer to this array to the function argument.
// (Use HG_GetNumberOfProgs() to get the array's length.)
//
// Input:  list - pointer to be set
//
// Output: HG_ERR_NO, HG_ERR_INVALID_ARG, HG_ERR_NOT_INIT,
//         HG_ERR_WRITING, EG_ERR_READING


RDSHG81300B_API int __stdcall  HG_GetNumberOfProgs  (void);
//
// returns the number of occupied 'program slots' of the RDS-coder.
//
// Input:  -
//
// Output: >=0: number of programs stored, 0...HG_SLOT_NUMBER_MAX
//         HG_ERR_NOT_INIT


RDSHG81300B_API int __stdcall  HG_GetFreeMemory  (void);
//
// returns the free data memory of the RDS-coder in bytes.
// The maximum is HG_MEMORY_MAX_BYTES. (One RDS group needs 13 bytes.)
//
// Input:  -
//
// Output: >=0: free memory, 0...HG_MEMORY_MAX_BYTES
//         HG_ERR_NOT_INIT


RDSHG81300B_API int __stdcall  HG_SetFreq  (enum enmHG_switches s,
                                            double value);
//
// sets the value of frequency A if s is set to HG_FREQ_A or
// sets the value of frequency B if s is set to HG_FREQ_B.
// The given value in MHz may vary in the range between
// HG_FREQ_MIN and HG_FREQ_MAX in steps of 0.1 MHz.
// The RDS-coder switches to this frequency automatically.
//
// Input:  s     - HG_FREQ_A or HG_FREQ_B
//         value - HG_FREQ_MIN...HG_FREQ_MAX
//
// Output: HG_ERR_NO, HG_ERR_INVALID_ARG, HG_ERR_NOT_INIT, HG_ERR_WRITING


RDSHG81300B_API int __stdcall  HG_SetSwitch  (enum enmHG_switches s,
                                              bool state);
//
// turns the switch s of the RDS-coder on or off.
//
// Input:  s     - switch, defined in enmHG_switches
//         state - false = off, true = on
//
// Output: HG_ERR_NO, HG_ERR_INVALID_ARG, HG_ERR_NOT_INIT, HG_ERR_WRITING


RDSHG81300B_API int __stdcall  HG_SelectProg  (int slot);
//
// switches over to the 'program slot' to be sent by the RDS-coder.
//
// The slot number currently sent is displayed in binary coded form
// by the 6 LEDs at the upper right of the panel.
// For example: o*oo** (bin. 010011) symbolizes slot no. 19.
//
// Input:  slot - HG_SLOT_NUMBER_MIN...HG_SLOT_NUMBER_MAX
//                or HG_NO_SLOT to send nothing
//
// Output: HG_ERR_NO, HG_ERR_INVALID_ARG, HG_ERR_NOT_INIT,
//         HG_ERR_WRITING, HG_ERR_READING, HG_ERR_CHECK_SEL


RDSHG81300B_API int __stdcall  HG_StoreProg  
                                 (const void *data,
                                  int groups,
                                  int slot,
                                  int times        = HG_SLOT_TIMES_INFINITE,
                                  int nextslot     = HG_NO_SLOT,
                                  const char *name = NULL                  );
//
// stores a RDS program (a RDS data sequence) to a 'program slot' of the
// RDS-coder (13 bytes per RDS group, data bits inclusive CRC bits).
// 'data' points to the program in memory,
// 'groups' sets the length of the program as number of RDS groups.
// (One RDS group needs 13 bytes.)
// 'slot' sets the 'program slot' to store the program,
// 'times' defines how often the program gets sent there,
// 'nextslot' gives the 'program slot' to be jumped on after this one.
// If 'times' is set to HG_SLOT_TIMES_INFINITE 'nextslot' will be ignored.
// 'name' is a string for the program name (an identifier in the program list)
// The last three arguments may be missing.
// The storing procedure may take a while (ca. 2 sec per 1 KByte). 
// After storing, the program list will be updated.
// The program stays stored even after switching off the device.
//
// If a RDS program is already stored in the slot it will be overwritten.
// Nevertheless there must be enough memory space available for both programs
// in the RDS-coder. So if you want to overwrite a program but there is not
// enough memory space free for the new one, the 'program slot' has to be
// deleted before.
//
// Input:  data     - pointer to the RDS data sequence (bytes)
//         groups   - 1...HG_MEMORY_MAX_GROUPS
//         slot     - HG_SLOT_NUMBER_MIN...HG_SLOT_NUMBER_MAX
//         times    - 1...HG_SLOT_TIMES_MAX, HG_SLOT_TIMES_INFINITE
//         nextslot - HG_SLOT_NUMBER_MIN...HG_SLOT_NUMBER_MAX, HG_NO_SLOT
//         name     - pointer to the character string
//                    (up to 8 characters, no space character allowed),
//                    if NULL, "########" is used as name
//
// Output: HG_ERR_NO, HG_ERR_INVALID_ARG, HG_ERR_NOT_INIT, HG_ERR_MEMORY,
//         HG_ERR_WRITING, HG_ERR_READING, HG_ERR_CHECK_STORE


RDSHG81300B_API int __stdcall  HG_DeleteProg  (int slot);
//
// deletes the contents of a 'program slot' if there is a program stored.
// After deleting, the program list will be updated.
//
// Input:  slot - HG_SLOT_NUMBER_MIN...HG_SLOT_NUMBER_MAX
//
// Output: HG_ERR_NO, HG_ERR_INVALID_ARG, HG_ERR_NOT_INIT,
//         HG_ERR_WRITING, HG_ERR_READING, HG_ERR_CHECK_SEL, HG_ERR_CHECK_DEL


RDSHG81300B_API int __stdcall  HG_FirmwareVersion  (char **str);
//
// sets a pointer to the string containing the firmware version of the
// RDS-coder.
//
// Input:  pointer to string pointer
//
// Output: HG_ERR_NO, HG_ERR_INVALID_ARG, HG_ERR_NOT_INIT


RDSHG81300B_API char* __stdcall  HG_ErrorMsg  (int result, int lang = 0);
//
// returns a short error message corresponding to the error code that is
// given in result.
// The default language is English. If lang is 1, the language is English.
// If 2 is given the messages are in German.
//
// Input:  result - error code (negative number, see constants "HG_ERR_x")
//         lang   - 0: no change, 1: English, 2: German
//
// Output: pointer to the character string


#endif // !defined(RDSHG81300B_H_INCLUDED)

//EOF